!------------------------------------------------------------------------------------!
! David Julian Kling | 2016 | Johann Wolfgang Goethe-Universität Frankfurt am Main   !
!------------------------------------------------------------------------------------!

module ODE_system
use params
implicit none


contains
      function rho(P_in)
      ! Zustandsgleichung
        real(kra) :: rho, P_in

        rho = (P_in/K)**(one/gamma_poly)  ! Polytrope Zustandsgleichung.
      end function rho


      function P(rho_in)
      ! Zustandsgleichung invertiert, benötigt zur Bestimmung von P_init = P(rho_init)
        real(kra) :: rho_in, P

        P = K*rho_in**gamma_poly  ! Polytrope Zustandsgleichung.
      end function P



      function dm_dr(r_n, rho_n)
        real(kra) :: dm_dr
        real(kra) :: r_n, rho_n

        dm_dr = four*pi*r_n*r_n*rho_n
      end function dm_dr

      function dP_dr(r_n,P_n, rho_n, m_n)
        real(kra) :: dP_dr
        real(kra) :: r_n, rho_n, m_n, P_n

        if( (r_n.ne.zero).and.(m_n.ne.zero) ) then
          dP_dr = -rho_n*m_n/(r_n*r_n) &
           *( one+P_n/rho_n )*(one + four*P_n*r_n*r_n*r_n/m_n )/(one-two*m_n/r_n) ! auskommentieren
                                                                       ! für Newton'schen Grenzfall
        else
          dP_dr = zero
        end if
      end function dP_dr

end module ODE_system



