!------------------------------------------------------------------------------------!
! David Julian Kling | 2016 | Johann Wolfgang Goethe-Universität Frankfurt am Main   !
!------------------------------------------------------------------------------------!




module rk4
! Runge-Kutta Algorithmus vierter Ordnung
use params
implicit none

contains
!-------------------------------------------------------------------------------!
! Koeffizienten für Runge-Kutta Algorithmus vierter Ordnung | Für (n+1)-Schritt !
!-------------------------------------------------------------------------------!
      function c4_m(r_n, rho_n, stepsize) result(res)
      use ODE_system, only : dm_dr
        real(kra) :: c4_m1, c4_m2, c4_m3, c4_m4 
        real(kra) :: res 
        real(kra) :: r_n, rho_n, stepsize

        !c_1 = stepsize * f(x_n,y_n), wobei f = y' und x unabh. variable z.B. "t", oder wie hier "r"
        !c_2 = stepsize * f(x_n+stepsize/2, y_n + c_1/ + c_1/2)
        !c_3 = ...
        !c_4 = ...

        c4_m1 = stepsize*dm_dr( r_n,                rho_n )
        c4_m2 = stepsize*dm_dr( r_n + stepsize/two, rho_n ) 
        c4_m3 = stepsize*dm_dr( r_n + stepsize/two, rho_n ) 
        c4_m4 = stepsize*dm_dr( r_n + stepsize,     rho_n )           
        res  = ( c4_m1+two*c4_m2+two*c4_m3+c4_m4 )/six
      end function c4_m

      function c4_P(r_n, rho_n, m_n, P_n, stepsize) result(res)
      use ODE_system, only : dP_dr
        real(kra) :: c4_P1, c4_P2, c4_P3, c4_P4
        real(kra) :: res 
        real(kra) :: r_n, m_n, P_n, rho_n, stepsize

        c4_P1 = stepsize*dP_dr( r_n,                P_n,             rho_n, m_n )
        c4_P2 = stepsize*dP_dr( r_n + stepsize/two, P_n + c4_P1/two, rho_n, m_n ) 
        c4_P3 = stepsize*dP_dr( r_n + stepsize/two, P_n + c4_P2/two, rho_n, m_n ) 
        c4_P4 = stepsize*dP_dr( r_n + stepsize,     P_n + c4_P3,     rho_n, m_n )           
        res  = ( c4_P1+two*c4_P2+two*c4_P3+c4_P4 )/six
      end function c4_P

     !! function c4_Phi(r_n, stepsize) result(res)
     !! use ODE_system, only : dPhi_dr
     !! implicit none
     !!   real(kra), intent(out) :: c4_m1, c4_m2, c4_m3, c4_m4, res 
     !!   real(kra), intent(in)  :: r_n
     !!
     !!   c4_Phi1 = stepsize*dPhi_dr( r_n )
     !!   c4_Phi2 = stepsize*dPhi_dr( r_n + stepsize/two ) 
     !!   c4_Phi3 = stepsize*dPhi_dr( r_n + stepsize/two  ) 
     !!   c4_Phi4 = stepsize*dPhi_dr( r_n + stepsize )           
     !!   res = ( c4_Phi1+two*c4_Phi2+two*c4_P3+c4_Phi4 )/six
     !! end function c4_Phi

!---------------------------------------------------------------------------------------------------!
! Koeff. für Runge-Kutta Algorithmus fünfter Ordnung | Für (n+1)-Schritt| Für adaptive Schrittweite !
!---------------------------------------------------------------------------------------------------!

end module rk4





